# Generated by roxygen2: do not edit by hand

export(PerformScalarizing)
export(UpdateIdealPoint)
export(UpdateWorstPoint)
export(associate_to_niches)
export(calc_norm_pref_distance)
export(compute_niche_count)
export(compute_perpendicular_distance)
export(crowding_distance)
export(generate_reference_points)
export(generational_distance)
export(getCrowdingDistance)
export(getDummyFitness)
export(getFitness)
export(getMetrics)
export(getPopulation)
export(get_fixed_rowsum_integer_matrix)
export(get_nadir_point)
export(modifiedCrowdingDistance)
export(niching)
export(non_dominated_fronts)
export(nsga)
export(nsga2)
export(nsga3)
export(plot)
export(print)
export(progress)
export(reference_point_multi_layer)
export(rmoo)
export(rmooControl)
export(rmooMonitor)
export(rmoo_huxCrossover)
export(rmoo_lrSelection)
export(rmoo_spCrossover)
export(rmoo_tourSelection)
export(rmoo_uxCrossover)
export(rmoo_uxMutation)
export(rmoobin_Population)
export(rmoobin_huxCrossover)
export(rmoobin_lrSelection)
export(rmoobin_raMutation)
export(rmoobin_spCrossover)
export(rmoobin_tourSelection)
export(rmoobin_uxCrossover)
export(rmoobin_uxMutation)
export(rmooint_Population)
export(rmooint_huxCrossover)
export(rmooint_spCrossover)
export(rmooint_uxCrossover)
export(rmooint_uxMutation)
export(rmooperm_Population)
export(rmooperm_lrSelection)
export(rmooperm_oxCrossover)
export(rmooperm_simMutation)
export(rmooperm_tourSelection)
export(rmooreal_Population)
export(rmooreal_lrSelection)
export(rmooreal_polMutation)
export(rmooreal_raMutation)
export(rmooreal_sbxCrossover)
export(rmooreal_spCrossover)
export(rmooreal_tourSelection)
export(rnsga2)
export(scale_reference_directions)
export(sharing)
export(startParallel)
export(stopParallel)
export(summary)
exportClasses(algorithm)
exportClasses(nsga)
exportClasses(nsga1)
exportClasses(nsga2)
exportClasses(nsga3)
exportClasses(numberOrNAOrMatrix)
exportClasses(rnsga2)
import(foreach)
import(graphics)
import(methods)
import(stats)
import(utils)
importFrom(BBmisc,which.first)
importFrom(GA,ga)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,label_value)
importFrom(grDevices,rainbow)
importFrom(plotly,"%>%")
importFrom(plotly,add_markers)
