#' @keywords internal
#' @aliases rmoo-package
"_PACKAGE"

#' @import stats utils graphics methods foreach
#' @importFrom GA ga
#' @importFrom grDevices rainbow
#' @importFrom ggplot2 aes
#' @importFrom ggplot2 geom_line
#' @importFrom ggplot2 label_value
#' @importFrom ggplot2 element_text
#' @importFrom plotly %>%
#' @importFrom plotly add_markers
#' @importFrom BBmisc which.first
NULL
