#' KROA100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroA100"

#' KROB100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroB100"


#' KROC100
#'
#' A dataset containing the coord and section of 100 cities
#'
#' @format A data frame with 100 rows and 2 variables:
#' \describe{
#'     \item{COORD}{City Coordinates}
#'     \item{SECTION}{City Section}
#' }
#' @references Reinelt, G. (1991). TSPLIB—A traveling salesman problem library.
#' ORSA journal on computing, 3(4), 376-384
"kroC100"
