% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel.R
\name{stopParallel}
\alias{stopParallel}
\title{Stop Parallel Backend}
\usage{
stopParallel(cluster, ...)
}
\arguments{
\item{cluster}{A cluster object, typically retrieved from \code{attr(parallel, "cluster")}}

\item{...}{Additional arguments (currently unused).}
}
\value{
\code{invisible(NULL)}, used for side effects.
}
\description{
Stops the parallel backend and reverts to sequential execution.
}
\examples{
\dontrun{
  cl <- startParallel()
  stopParallel(attr(cl, "cluster"))
}

}
